@@ -11,7 +11,7 @@ from typing import Dict, Any
 from queue import Queue, Empty
 from contextlib import contextmanager
 from pyngrok import ngrok, conf as ngrokconf
-from concurrent.futures import ThreadPoolExecutor, CancelledError
+from concurrent.futures import ThreadPoolExecutor
 #
 from autogram import *
 from autogram.webhook import MyServer
@@ -22,27 +22,44 @@ class Autogram:
     api_url = 'https://api.telegram.org/'
 
     def __init__(self, config: Dict):
+        self.token :str|None = config.get('telegram-token')
+        if not self.token:
+            print('Missing bot token!')
+            sys.exit(1)
+        #
         self.config = config
         self._initialize_()
 
     def _initialize_(self):
         self.webhook = None
         self.host = '0.0.0.0'
+        self.deleted_ = set()
+        self.public_ip = None
         self.update_offset = 0
+        self.seenUpdates= list()
+        self.ngrok_tunnel = None
         self.httpRoutines = list()
         self.httpRequests = Queue()
-        self.worker_threads = {
-            'io-tasks': list(),
-            'common-tasks': list(),
-            'high-priority': list(),
-        }
         self.failing_endpoints = list()
         self.terminate = threading.Event()
         self.port = self.config['tcp-port']
-        self.control_lock = threading.Lock()
         self.executor = ThreadPoolExecutor()
+        self.base_url = f"{Autogram.api_url}bot{self.token}"
         self.timeout = aiohttp.ClientTimeout(self.config['tcp-timeout'])
-        self.base_url = f"{Autogram.api_url}bot{self.config['telegram-token']}"
+        #
+        self.guard = {
+            'lock': threading.Lock(),
+            'pending': Queue(),
+            'thread': None
+        }
+        #
+        self.worker_threads = {
+            'normal': list(),
+            'high': list(),
+        }
+        self.locks = {
+            'session': threading.Lock(),
+        }
         # 
         logger_format = (
             "<green>{time:DD/MM/YYYY HH:mm:ss}</green> | "
@@ -59,8 +76,7 @@ class Autogram:
         #
         loguru.logger.add(sys.stderr, format=logger_format, level=lvl)
         self.logger = loguru.logger
-        self.control_lock.acquire()
-        self.logger.debug('Logs initialized successfully')
+        return
 
     def mediaQuality(self):
         if (qlty := self.config.get("media-quality") or 'high') == 'high':
@@ -72,8 +88,10 @@ class Autogram:
     @loguru.logger.catch
     def send_online(self) -> threading.Thread:
         """Get this bot online in a separate daemon thread."""
+        if not self.token:
+            raise RuntimeError("Send online without token?!")
         if public_ip := self.config['tcp-ip']:
-            hookPath = self.config['telegram-token'].split(":")[-1].lower()[:8]
+            hookPath = self.token.split(":")[-1].lower()
             @post(f'/{hookPath}')
             def hookHandler():
                 self.updateRouter(request.json)
@@ -92,23 +110,22 @@ class Autogram:
             if public_ip == 'ngrok':
                 ngrok_config = ngrokconf.PyngrokConfig(
                     ngrok_version='v3',
-                    ngrok_path='/snap/bin/ngrok',
-                    auth_token= self.config.get('ngrok-token')
+                    ngrok_path= self.config.get('ngrok-path'),
+                    auth_token= self.config.get('ngrok-token'),
+                    config_path= self.config.get('ngrok-config-path')
                 )
                 ngrokconf.set_default(ngrok_config)
-                if log_ngrok := self.config.get('ngrok-logs'):
-                    ngrokconf.get_default().log_event_callback = self.logger.debug
                 try:
                     self.ngrok_tunnel = ngrok.connect()
-                    if not log_ngrok:
-                        ngrok.get_ngrok_process().stop_monitor_thread()
+                    ngrok.get_ngrok_process().stop_monitor_thread()
                     public_ip = self.ngrok_tunnel.public_url
                 except Exception as e:
                     self.logger.critical(e)
-                    self.terminate.set()
+                    self.shutdown()
             #
+            self.public_ip = public_ip
             if not self.terminate.is_set():
-                self.webhook = f"{public_ip}/{hookPath}"
+                self.webhook = f"{self.public_ip}/{hookPath}"
                 self.logger.debug(f'Webhook: {self.webhook}')
         # wrap and start main_loop
         def launch():
@@ -119,15 +136,15 @@ class Autogram:
                     asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
                 asyncio.run(self.main_loop())
             except KeyboardInterrupt:
-                self.terminate.set()
+                self.shutdown()
             except Exception as e:
-                loguru.logger.exception(e)
+                self.logger.exception(e)
             finally:
-                self.terminate.set()
+                if not self.terminate.is_set():
+                    self.shutdown()
         #
         worker = threading.Thread(target=launch)
         worker.name = 'Autogram'
-        worker.daemon = True
         worker.start()
         return worker
 
@@ -145,21 +162,24 @@ class Autogram:
             }, None))
         else:   # delete webhook
             def check_webhook(info: dict):
-                if info['url']:
-                    self.deleteWebhook()
+                if url := info['url']:
+                    self.logger.debug(f'Webhook deleted: {url}')
+            self.deleteWebhook()
             self.getWebhookInfo(check_webhook)
         #
         await asyncio.sleep(0)    # allow getMe to run
         await processor
-        #
-        self.terminate.set()
-        self.logger.info('Autogram terminated.')
+        return
 
     @loguru.logger.catch
     def updateRouter(self, res: Any):
         """receive and route updates"""
         def handle(update: Dict):
+            if (uid := update['update_id']) in self.seenUpdates:
+                return
+            #
             parser = None
+            self.seenUpdates.append(uid)
             # parse update
             if payload:=update.get(Message.name):
                 if payload['chat']['type'] == 'private':
@@ -196,7 +216,8 @@ class Autogram:
             if not parser:
                 return
             # todo: implement all update types then allow through
-            if parser.name != 'message':
+            done_routes = ['message', 'callback_query']
+            if parser.name not in done_routes:
                 self.logger.critical(f"Unimplemented: {parser.name}")
                 return
             #
@@ -212,50 +233,128 @@ class Autogram:
         handle(res)
         return
 
+    def popWorkers(self, workers: list):
+        for item in workers:
+            key, task = item
+            self.worker_threads[key].remove(task)
+            continue
+        return
+
+    @loguru.logger.catch
+    def threadGuard(self):
+        workers = 0
+        to_remove = list()
+        self.guard['lock'].acquire()
+        while not self.terminate.is_set():
+            try:
+                worker = self.guard['pending'].get(timeout=3)
+                priority, work = worker
+                #
+                self.worker_threads[priority].append(work)
+                workers += 1
+            except Empty:
+                continue
+            #
+            for key in self.worker_threads:
+                for task in self.worker_threads[key]:
+                    #
+                    _ , tsk, cb, errh = task
+                    if not tsk.done():
+                        continue
+                    # fetch results or exceptions
+                    try:
+                        result = tsk.result()
+                        if cb:
+                            self.toThread(cb, result)
+                    except Exception as e:
+                        if errh:
+                            errh(e)
+                        else:
+                            self.logger.exception(e)
+                    to_remove.append((key, task))
+                    workers -= 1
+            #
+            self.popWorkers(to_remove)
+            to_remove.clear()
+        # shutdown executor
+        if not workers:
+            self.guard['lock'].release()
+            return
+        elif self.worker_threads['high'] or self.worker_threads['normal']:
+            self.logger.debug("Terminating worker threads...")
+            #
+            for task in self.worker_threads['normal']:
+                tsk_id, tsk, cb, _ = task
+                if not tsk.done() and not tsk.running():
+                    if tsk.cancel():
+                        self.logger.debug(f"[{tsk_id}] canceled.")
+                    else:
+                        self.logger.debug(f"[{tsk_id}] rejected cancel request.")
+                elif done := tsk.done() and cb and not tsk.exception():
+                    self.logger.debug(f"[{tsk_id}] completed but was ignored on termination.")
+                elif done and (e := tsk.exception()):
+                    self.logger.exception(e)
+                continue
+            #
+            for worker in self.worker_threads['high']:
+                tsk_id, tsk, cb, _ = worker
+                if tsk.done():
+                    to_remove.append(('high',worker))
+                    # propagate result
+                    if cb and not tsk.exception():
+                        cb(tsk.result())
+                    elif e := tsk.exception():
+                        self.logger.exception(e)
+                        self.logger.debug(f"[{tsk_id}] finished with an error.")
+                elif not tsk.running():
+                    if tsk.cancel():
+                        self.logger.debug(f"[{tsk_id}] : Not started! Canceled.")
+                    else:
+                        self.logger.debug(f"[{tsk_id}] : Cancel failed!")
+                else:
+                    self.logger.debug(f"[{tsk_id}] : thread is busy!")
+                continue
+            self.executor.shutdown(wait=False, cancel_futures=True)
+        #
+        self.logger.debug("Tasks terminated.")
+        self.guard['lock'].release()
+        return
+
     @loguru.logger.catch()
-    def toThread(self, *args, callback: Callable = None, errHandler: Callable = None, priority : str = ''):
-        if self.terminate.is_set():
-            self.logger.debug(f"[{callable.__name__}] : Skipping execution on termination")
+    def toThread(self, *args, callback :Callable|None = None, errHandler :Callable|None = None, priority :str|None = None):
+        if self.locks['session'].locked():
+            self.logger.debug(f"[{args[0].__name__}] : Blocked execution")
             return
         #
-        to_remove = list()
-        for key in self.worker_threads:
-            for task in self.worker_threads[key]:
-                tsk, cb, errh = task
-                if not tsk.done():
-                    continue
-                # fet results or exceptions
-                try:
-                    result = tsk.result()
-                    if cb:
-                        cb(result)
-                except Exception as e:
-                    if errh:
-                        errh(e)
-                    else:
-                        self.logger.exception(e)
-                to_remove.append((key, self.worker_threads[key].index(task)))
-        #
-        for item in to_remove:
-            key, index = item
-            self.worker_threads[key].pop(index)
-            continue
-        # append workers to priority groups
+        if not self.guard['lock'].locked():
+            self.guard['thread'] = threading.Thread(target=self.threadGuard)
+            self.guard['thread'].name = 'Autogram::ThreadGuard'
+            self.guard['thread'].start()
+        # append worker to priority group
         result = False
         if not priority or (result := (priority not in list(self.worker_threads.keys()))):
             if result:
-                self.logger.debug(f"Undefined task priority: {priority}")
-            priority = 'common-tasks'
-        task = self.executor.submit(*args)
-        self.worker_threads[priority].append((task, callback, errHandler))
-        return task
+                self.logger.debug(f"Undefined task priority: {priority}: {args}")
+            priority = 'normal'
+        #
+        tsk_id = args[0].__name__
+        try:
+            task = self.executor.submit(*args)
+            self.guard['pending'].put((priority, (tsk_id, task, callback, errHandler)))
+            return task
+        except RuntimeError:
+            self.terminate.set()
+        return
 
     @contextmanager
     def get_request(self):
         """fetch pending or failed task from tasks"""
         if self.failed:
-            self.logger.info('Retrying failed request...')
-            yield self.failed
+            prev = self.failed
+            self.failed = None
+            yield prev
+            if prev == self.failed:
+                time.sleep(2)
             return
         elif self.webhook:
             if not self.terminate.is_set():
@@ -277,54 +376,57 @@ class Autogram:
             yield self.httpRequests.get(block=False)
             return
         yield None
+        return
+
+    @loguru.logger.catch
+    async def httpHandler(self):
+        while not self.terminate.is_set():
+            if not self.httpRoutines:
+                await asyncio.sleep(1)
+                continue
+            #
+            for item in self.httpRoutines:
+                if self.terminate.is_set():
+                    return
+                #
+                incoming, outgoing = item
+                resp, payload = incoming
+                link, _, callback = outgoing
+                endpoint = link.split('/')[-1]
+                #
+                if resp.ok:
+                    if endpoint in self.failing_endpoints:
+                        self.failing_endpoints.remove(endpoint)
+                    #
+                    if (data := payload.get('result')) and callback:
+                        self.toThread(callback, data)
+                    elif self.config.get('echo-responses'):
+                            self.logger.debug(payload)
+                    continue
+                elif resp.status == 401:
+                    self.logger.critical("Invalid token. Closing...")
+                    self.shutdown()
+                elif endpoint not in self.failing_endpoints:
+                    if payload:
+                        self.logger.critical(f"[{endpoint}] HTTP{resp.status} : {endpoint} : {payload}")
+                    else:
+                        self.logger.critical(f"[{endpoint}] HTTP{resp.status} : {outgoing}")
+                # ignore repeated output
+                self.failing_endpoints.append(endpoint)
+                continue
+            # clear handled routines
+            self.httpRoutines.clear()
 
     @loguru.logger.catch()
     async def aioWebRequest(self):
         """Make asynchronous requests to the Telegram API"""
         async with aiohttp.ClientSession(timeout=self.timeout) as session:
             self.failed = None
+            asyncio.create_task(self.httpHandler())
             #
-            @loguru.logger.catch
-            async def httpHandler():
-                while not self.terminate.is_set():
-                    if not self.httpRoutines:
-                        await asyncio.sleep(0)
-                        continue
-                    #
-                    for item in self.httpRoutines:
-                        incoming, outgoing = item
-                        #
-                        resp, payload = incoming
-                        _, _, callback = outgoing
-                        endpoint = link.split('/')[-1]
-                        if resp.ok:
-                            if endpoint in self.failing_endpoints:
-                                self.failing_endpoints.remove(endpoint)
-                            self.httpRoutines.remove(item)
-                            #
-                            if payload and callback:
-                                self.toThread(callback, payload)
-                                # if it was a getMe request, wait for it to finish
-                                if not self.control_lock.locked():
-                                    self.control_lock.acquire()
-                            continue
-                        elif endpoint not in self.failing_endpoints:
-                            if payload:
-                                self.logger.critical(f"[{endpoint}] HTTP{resp.status} : {endpoint} : {payload}")
-                            else:
-                                self.logger.critical(f"[{endpoint}] HTTP{resp.status} : {outgoing}")
-                        # ignore repeated output
-                        self.failing_endpoints.append(endpoint)
-                        continue
-                    # clear handled routines
-                    self.httpRoutines.clear()
-            #
-            # start httpHandler
-            asyncio.create_task(httpHandler())
             while not self.terminate.is_set():
                 with self.get_request() as request:
                     if not request:
-                        ## no webhook? convert to getJob task
                         if not self.webhook:
                             params = {
                                 'params': {
@@ -332,16 +434,18 @@ class Autogram:
                                 }
                             }
                             url = f'{self.base_url}/getUpdates'
-                            request = (url,params,self.updateRouter)
-                        else:
-                            continue
+                            request = (url, params, self.updateRouter)
+                            self.httpRequests.put(request)
+                        await asyncio.sleep(0)
+                        continue
+                    #
                     link, kw, _ = request
                     kw = kw or dict()
                     defaults = {
                         'params': {
                             "limit": 81,
                             "offset": self.update_offset,
-                            "timeout": self.timeout.total - 1,
+                            "timeout": self.timeout.total,
                         }
                     }
                     if not kw.get('params'):
@@ -349,35 +453,35 @@ class Autogram:
                     else:
                         kw['params'] |= defaults['params']
                     ##
+                    if self.terminate.is_set():
+                        return
+                    #
+                    error_detected = None
                     try:
-                        error_detected = None
                         async with session.get(link,**kw) as resp:
                             data = None
                             if resp.ok:
                                 data = await resp.json()
-                                self.logger.debug(f'GET [ok] /{link.split("/")[-1]}')
-                            else:
-                                self.logger.debug(f'GET [err] /{link.split("/")[-1]} : {kw}')
+                            self.logger.debug(f'[{resp.status}] GET /{link.split("/")[-1]}')
                             self.httpRoutines.append(((resp, data), request))
+                            await asyncio.sleep(0)
                     except KeyboardInterrupt:
                         self.terminate.set()
-                        self.logger.critical('Termination with ^C')
                     except aiohttp.ClientConnectorError as e:
                         error_detected = e
-                        self.logger.critical('ClientConnectorError')
                     except aiohttp.ClientOSError as e:
                         error_detected = e
-                        self.logger.critical('ClientOSError')
                     except asyncio.TimeoutError as e:
                         error_detected = e
-                        self.logger.critical('TimeoutError')
+                    except RuntimeError as e:
+                        error_detected = e
                     except Exception as e:
                         error_detected = e
-                        self.logger.critical(f'Unknown Error: {e}')
+                        self.terminate.set()
+                        self.logger.exception(e)
                     finally:
                         if error_detected:
                             self.failed = request
-                            self.logger.exception(error_detected)
                 #
                 await asyncio.sleep(0)
 
@@ -399,40 +503,44 @@ class Autogram:
         finally:
             return False, res, url
 
-    def shutdown(self, callback: Callable = None):
+    def shutdown(self, callback :Callable|None = None):
         """callback: your exit function that takes `msg : str`"""
-        self.terminate.set()
-        exit_msg = list()
-        critical_priority = ['io-threads', 'high-priority']
-        for key in critical_priority:
-            for task in self.worker_threads[key]:
-                tsk, cb, errh = task
-                if not tsk.done() and errh:
-                    if tsk.cancel():
-                        errh(CancelledError(f'Thread leading to {cb.__name__} was canceled'))
-                    else:
-                        errh(CancelledError(f'Thread leading to {cb.__name__} rejected termination.'))
-                elif not tsk.done():
-                    exit_msg.append(f"thread to: {cb.__name__} ~ busy")
+        if self.locks['session'].locked() or self.terminate.is_set():
+            return
+        # block further updates
+        try:
+            ngrok.disconnect(self.public_ip)
+        except:
+            pass
         #
-        msg = '\n'.join(exit_msg)
+        # start termination
+        self.terminate.set()
         if callback:
-            callback(msg)
-        else:
-            self.logger.debug(msg)
+            try:
+                callback()
+            except Exception as e:
+                self.logger.exception(e)
+        # prevent toThread in subsequent calls
+        if not self.locks['session'].locked():
+            self.locks['session'].acquire()
+        # terminate and wait for threadGuard
+        self.logger.info('Autogram::terminating...')
+        self.terminate.set()
+        if self.guard['lock'].locked():
+            self.guard['thread'].join()
+        #
+        self.logger.info('Autogram::terminated.')
+        return
 
     #***** start API calls *****#
     def getMe(self, me: Dict):
         """receive and parse getMe request."""
-        self.logger.info('*** connected... ***')
         for k,v in me.items():
             setattr(self, k, v)
-        # 
-        self.control_lock.release()
 
     @loguru.logger.catch()
     def downloadFile(self, file_path: str):
-        url = f"https://api.telegram.org/file/bot{self.config['telegram-token']}/{file_path}"
+        url = f"https://api.telegram.org/file/bot{self.token}/{file_path}"
         res = requests.get(url)
         if res.ok:
             return res.content
@@ -467,17 +575,23 @@ class Autogram:
         return self.webRequest(f'{self.base_url}/sendChatAction', params=params)
 
     @loguru.logger.catch()
-    def sendMessage(self, chat_id: int, text: str, **kwargs):
+    def sendMessage(self, chat_id :int|str, text :str, **kwargs):
+        callback = None
+        if 'callback' in kwargs.keys():
+            callback = kwargs.pop('callback')
         url = f'{self.base_url}/sendMessage'
         self.httpRequests.put((url,{
             'params': {
                 'chat_id': chat_id,
                 'text': text,
             } | kwargs
-        } , None))
+        } , callback))
 
     @loguru.logger.catch()
     def deleteMessage(self, chat_id: int, msg_id: int):
+        if msg_id in self.deleted_:
+            return False
+        self.deleted_.add(msg_id)
         url = f'{self.base_url}/deleteMessage'
         self.httpRequests.put((url,{
             'params': {
@@ -485,6 +599,7 @@ class Autogram:
                 'message_id': msg_id
             }
         }, None))
+        return True
 
     @loguru.logger.catch()
     def deleteWebhook(self):
@@ -492,15 +607,19 @@ class Autogram:
         return self.webRequest(url)
 
     @loguru.logger.catch()
-    def editMessageText(self, chat_id: int, msg_id: int, text: str, params={}):
+    def editMessageText(self, chat_id: int, msg_id: int, text: str, **kwargs):
+        if msg_id in self.deleted_:
+            return False
+        #
         url = f'{self.base_url}/editMessageText'
-        self.httpRequests.put((url,{
+        self.httpRequests.put((url, {
             'params': {
                 'text':text,
                 'chat_id': chat_id,
                 'message_id': msg_id
-            }|params
+            } | kwargs
         },None))
+        return True
 
     @loguru.logger.catch()
     def editMessageCaption(self, chat_id: int, msg_id: int, capt: str, params={}):
@@ -515,6 +634,9 @@ class Autogram:
 
     @loguru.logger.catch()
     def editMessageReplyMarkup(self, chat_id: int, msg_id: int, markup: str, params={}):
+        if msg_id in self.deleted_:
+            return False
+        #
         url = f'{self.base_url}/editMessageReplyMarkup'
         self.httpRequests.put((url,{
             'params': {
@@ -523,6 +645,7 @@ class Autogram:
                 'reply_markup': markup
             }|params
         }, None))
+        return True
 
     @loguru.logger.catch()
     def forwardMessage(self, chat_id: int, from_chat_id: int, msg_id: int):
@@ -536,16 +659,18 @@ class Autogram:
         },None))
 
     @loguru.logger.catch()
-    def answerCallbackQuery(self, query_id,text:str= None, params : dict= None):
+    def answerCallbackQuery(self, query_id, text :str|None = None, params : dict|None = None):
         url = f'{self.base_url}/answerCallbackQuery'
+        params = params or {}
+        text = text or ''
         params.update({
             'callback_query_id':query_id,
-            'text':text
+            'text': text[:199] # 200 max characters
         })
         return self.webRequest(url, params)
 
     @loguru.logger.catch()
-    def sendPhoto(self,chat_id: int, photo_bytes: bytes, caption: str= None, params: dict= None):
+    def sendPhoto(self,chat_id: int, photo_bytes: bytes, caption: str|None = None, params: dict|None = None):
         params = params or {}
         url = f'{self.base_url}/sendPhoto'
         params.update({
@@ -556,7 +681,7 @@ class Autogram:
         return self.webRequest(url,params=params,files={'photo':photo_bytes})
 
     @loguru.logger.catch()
-    def sendAudio(self,chat_id: int,audio_bytes: bytes, caption: str= None, params: dict= None):
+    def sendAudio(self,chat_id: int,audio_bytes: bytes, caption: str|None = None, params: dict|None = None):
         params = params or {}
         url = f'{self.base_url}/sendAudio'
         params.update({
@@ -567,7 +692,7 @@ class Autogram:
         return self.webRequest(url,params,files={'audio':audio_bytes})
 
     @loguru.logger.catch()
-    def sendDocument(self,chat_id: int ,document_bytes: bytes, caption: str= None, params: dict= None):
+    def sendDocument(self,chat_id: int ,document_bytes: bytes, caption: str|None = None, params: dict|None = None):
         params = params or {}
         url = f'{self.base_url}/sendDocument'
         params.update({
@@ -578,7 +703,7 @@ class Autogram:
         return self.webRequest(url,params,files={'document':document_bytes})
 
     @loguru.logger.catch()
-    def sendVideo(self,chat_id: int ,video_bytes: bytes, caption: str = None, params: dict= None ):
+    def sendVideo(self,chat_id: int ,video_bytes: bytes, caption: str|None = None, params: dict|None = None ):
         params = params or {}
         url = f'{self.base_url}/sendVideo'
         params.update({
@@ -589,7 +714,7 @@ class Autogram:
         return self.webRequest(url,params,files={'video':video_bytes})
 
     @loguru.logger.catch()
-    def forceReply(self, params: dict= None):
+    def forceReply(self, params: dict|None = None):
         params = params or {}
         markup = {
             'force_reply': True,
@@ -597,7 +722,7 @@ class Autogram:
         return json.dumps(markup)
 
     @loguru.logger.catch()
-    def getKeyboardMarkup(self, keys: list, params: dict= None):
+    def getKeyboardMarkup(self, keys: list, params: dict|None = None):
         params = params or {}
         markup = {
             "keyboard":[row for row in keys]
@@ -605,7 +730,7 @@ class Autogram:
         return json.dumps(markup)
 
     @loguru.logger.catch()
-    def getInlineKeyboardMarkup(self, keys: list, params: dict= None):
+    def getInlineKeyboardMarkup(self, keys: list, params: dict|None = None):
         params = params or {}
         markup = {
             'inline_keyboard':keys
@@ -613,12 +738,13 @@ class Autogram:
         return json.dumps(markup)
 
     @loguru.logger.catch()
-    def parseFilters(self, filters: dict= None):
+    def parseFilters(self, filters: dict|None = None):
+        filters = filters or {}
         keys = list(filters.keys())
         return json.dumps(keys)
 
     @loguru.logger.catch()
-    def removeKeyboard(self, params: dict= None):
+    def removeKeyboard(self, params: dict|None = None):
         params = params or {}
         markup = {
             'remove_keyboard': True,
