@@ -1,3 +1,4 @@
+from re import T
 import loguru
 from . import UpdateBase
 from typing import Dict, Callable
@@ -13,12 +14,16 @@ class Message(UpdateBase):
     }
 
     @classmethod
-    def getCommands(cls):
-        """Get a list of commands with their function names"""
+    def getCommands(cls, raw =True):
+        """Get a list of commands or (command, callbackName) tuples"""
+        if raw:
+            return list(cls.endpoints['command-endpoint'])
+        #
         out = list()
-        for val in cls.endpoints['command-endpoint']:
-            out.append(f"{val.strip('/')} - {cls.endpoints['command-endpoint'].get(val).__name__}")
-        return '\n'.join(out)
+        for key in cls.endpoints['command-endpoint']:
+            func = cls.endpoints['command-endpoint'][key]
+            out.append((key.strip('/'), func))
+        return out
 
     def __init__(self, update: Dict):
         self.logger = self.autogram.logger
@@ -47,7 +52,10 @@ class Message(UpdateBase):
                 return
             #
             if handler := self.endpoints[endpoint].get(text):
-                handler(self)
+                try:
+                    handler(self)
+                except Exception as e:
+                    self.logger.exception(e)
             else:
                 self.delete()
                 self.autogram.sendMessage(
@@ -64,7 +72,10 @@ class Message(UpdateBase):
                 break
             if handler := self.endpoints[endpoint].get(key):
                 setattr(self, key, self.attachments.get(key))
-                handler(self)
+                try:
+                    handler(self)
+                except Exception as e:
+                    self.logger.exception(e)
             continue
 
     def __repr__(self):
@@ -128,5 +139,8 @@ class Message(UpdateBase):
                 'bytes': content
             }|file_info
             if handler := self.endpoints['user-endpoint'].get(key):
-                handler(self)
+                try:
+                    handler(self)
+                except Exception as e:
+                    self.logger.exception(e)
 
